/**
 **   xpcspeaker.c - xPC Target device driver for the internal speaker
 **
 **   Copyright 2008 The MathWorks, Inc.
 **/


#define S_FUNCTION_LEVEL   2
#undef  S_FUNCTION_NAME
#define S_FUNCTION_NAME    xpcspeaker

#include <stddef.h>
#include <stdlib.h>
#include <math.h>
#include "simstruc.h"

#ifndef MATLAB_MEX_FILE
#include "xpctarget.h"
#endif

#ifdef MATLAB_MEX_FILE
#include "mex.h"
#endif


void SoundOnOff(int state)
{
#ifndef MATLAB_MEX_FILE
   unsigned short int portval;

   portval = xpcInpB(0x61);

   if ( state ) portval |=  0x03;
           else portval &= ~0x03;

   xpcOutpB(0x61, portval);
#endif
}

void PlayNote(double frequency)
{
#ifndef MATLAB_MEX_FILE
   unsigned long int counter;

   counter = 1193180L / frequency;

   xpcOutpB(0x43, 0xB6);
   xpcOutpB(0x42, counter & 0xFF);
   xpcOutpB(0x42, counter >> 8);
#endif
}


static void mdlInitializeSizes(SimStruct *S)
{
   ssSetNumSFcnParams(S, 1);
   if ( ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S) )
        return;

   ssSetNumContStates(S, 0);
   ssSetNumDiscStates(S, 0);

   ssSetNumInputPorts(S, 1);
   ssSetInputPortDataType(S, 0, SS_DOUBLE);
   ssSetInputPortWidth(S, 0, 1);
   ssSetInputPortDirectFeedThrough(S, 0, 1);
   ssSetInputPortRequiredContiguous(S, 0, 1);

   ssSetNumOutputPorts(S, 0);

   ssSetNumIWork(S, 1);
   ssSetNumPWork(S, 0);
   ssSetNumRWork(S, 1);
   ssSetNumModes(S, 0);

   ssSetNumSampleTimes(S, 1);
   ssSetNumNonsampledZCs(S, 0);

   ssSetSFcnParamTunable(S, 0, SS_PRM_NOT_TUNABLE);

   ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE);
}

static void mdlInitializeSampleTimes(SimStruct *S)
{
   ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
   ssSetOffsetTime(S, 0, FIXED_IN_MINOR_STEP_OFFSET);
}

#define MDL_START
static void mdlStart(SimStruct *S)
{
   ssSetIWorkValue(S, 0, 0);
   SoundOnOff(0);

   ssSetRWorkValue(S, 0, 0.0);
}

static void mdlOutputs(SimStruct *S, int_T tid)
{
   real_T fToPlay;
   real_T resolution = (real_T)mxGetPr(ssGetSFcnParam(S, 0))[0];
   real_T *frequencyIn = ssGetInputPortRealSignal(S, 0);


   fToPlay = fabs(*frequencyIn);

   if ( fToPlay <= resolution ) {
      if ( ssGetIWorkValue(S, 0) ) {
         ssSetIWorkValue(S, 0, 0);
         SoundOnOff(0);
      }
   }
   else if ( (*frequencyIn < 0.0) ||
             (fabs(fToPlay - ssGetRWorkValue(S, 0)) > resolution) ) {
      if ( ! ssGetIWorkValue(S, 0) ) {
         ssSetIWorkValue(S, 0, 1);
         SoundOnOff(1);
      }
      PlayNote(fToPlay);
   }

   ssSetRWorkValue(S, 0, fToPlay);
}

static void mdlTerminate(SimStruct *S)
{
    SoundOnOff(0);
}


#ifdef MATLAB_MEX_FILE
#include "simulink.c"
#else
#include "cg_sfun.h"
#endif
